@@ -1,10 +1,18 @@
 //!The ini module provides all the things necessary to load and parse ini-syntax files. The most important of which is the `Ini` struct.
 //!See the [implementation](https://docs.rs/configparser/*/configparser/ini/struct.Ini.html) documentation for more details.
+#[cfg(feature = "indexmap")]
+use indexmap::IndexMap as Map;
+#[cfg(not(feature = "indexmap"))]
+use std::collections::HashMap as Map;
+
+#[cfg(feature = "async-std")]
+use async_std::{fs as async_fs, path::Path as AsyncPath};
+
 use std::collections::HashMap;
+use std::convert::AsRef;
 use std::fs;
-use std::fs::File;
-use std::io::prelude::*;
 use std::path::Path;
+use std::fmt::Write;
 
 ///The `Ini` struct simply contains a nested hashmap of the loaded configuration, the default section header and comment symbols.
 ///## Example
@@ -14,17 +22,127 @@ use std::path::Path;
 ///let mut config = Ini::new();
 ///```
 #[derive(Debug, Clone, Eq, PartialEq, Default)]
+#[non_exhaustive]
 pub struct Ini {
-    map: HashMap<String, HashMap<String, Option<String>>>,
+    map: Map<String, Map<String, Option<String>>>,
     default_section: std::string::String,
     comment_symbols: Vec<char>,
     delimiters: Vec<char>,
+    boolean_values: HashMap<bool, Vec<String>>,
     case_sensitive: bool,
+    multiline: bool,
 }
 
+///The `IniDefault` struct serves as a template to create other `Ini` objects from. It can be used to store and load
+///default properties from different `Ini` objects.
+///## Example
+///```rust
+///use configparser::ini::Ini;
+///
+///let mut config = Ini::new();
+///let default = config.defaults();
+///let mut config2 = Ini::new_from_defaults(default); // default gets consumed
+///```
+#[derive(Debug, Clone, Eq, PartialEq)]
+#[non_exhaustive]
+pub struct IniDefault {
+    ///Denotes the default section header name.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///
+    ///let mut config = Ini::new();
+    ///let default = config.defaults();
+    ///assert_eq!(default.default_section, "default");
+    ///```
+    pub default_section: std::string::String,
+    ///Denotes the set comment symbols for the object.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///
+    ///let mut config = Ini::new();
+    ///let default = config.defaults();
+    ///assert_eq!(default.comment_symbols, vec![';', '#']);
+    ///```
+    pub comment_symbols: Vec<char>,
+    ///Denotes the set delimiters for the key-value pairs.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///
+    ///let mut config = Ini::new();
+    ///let default = config.defaults();
+    ///assert_eq!(default.delimiters, vec!['=', ':']);
+    ///```
+    pub delimiters: Vec<char>,
+    pub boolean_values: HashMap<bool, Vec<String>>,
+    ///Denotes if the `Ini` object is case-sensitive.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///
+    ///let mut config = Ini::new();
+    ///let default = config.defaults();
+    ///assert_eq!(default.case_sensitive, false);
+    ///```
+    pub case_sensitive: bool,
+    ///Denotes if the `Ini` object parses multiline strings.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///
+    ///let mut config = Ini::new();
+    ///let default = config.defaults();
+    ///assert_eq!(default.multiline, false);
+    ///```
+    pub multiline: bool,
+}
+
+impl Default for IniDefault {
+    fn default() -> Self {
+        Self {
+            default_section: "default".to_owned(),
+            comment_symbols: vec![';', '#'],
+            delimiters: vec!['=', ':'],
+            multiline: false,
+            boolean_values: [
+                (
+                    true,
+                    vec!["true", "yes", "t", "y", "on", "1"]
+                        .iter()
+                        .map(|&s| s.to_owned())
+                        .collect(),
+                ),
+                (
+                    false,
+                    vec!["false", "no", "f", "n", "off", "0"]
+                        .iter()
+                        .map(|&s| s.to_owned())
+                        .collect(),
+                ),
+            ]
+            .iter()
+            .cloned()
+            .collect(),
+            case_sensitive: false,
+        }
+    }
+}
+
+#[cfg(windows)]
+const LINE_ENDING: &str = "\r\n";
+#[cfg(not(windows))]
+const LINE_ENDING: &str = "\n";
+
 impl Ini {
-    ///Creates a new `HashMap` of `HashMap<String, HashMap<String, Option<String>>>` type for the struct.
-    ///All values in the HashMap are stored in `String` type.
+    ///Creates a new `Map` of `Map<String, Map<String, Option<String>>>` type for the struct.
+    ///All values in the Map are stored in `String` type.
+    ///
+    ///By default, [`std::collections::HashMap`] is used for the Map object.
+    ///The `indexmap` feature can be used to use an [`indexmap::map::IndexMap`] instead, which
+    ///allows keeping the insertion order for sections and keys.
+    ///
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -33,17 +151,11 @@ impl Ini {
     ///```
     ///Returns the struct and stores it in the calling variable.
     pub fn new() -> Ini {
-        Ini {
-            map: HashMap::new(),
-            default_section: "default".to_owned(),
-            comment_symbols: vec![';', '#'],
-            delimiters: vec!['=', ':'],
-            case_sensitive: false,
-        }
+        Ini::new_from_defaults(IniDefault::default())
     }
 
-    ///Creates a new **case-sensitive** `HashMap` of `HashMap<String, HashMap<String, Option<String>>>` type for the struct.
-    ///All values in the HashMap are stored in `String` type.
+    ///Creates a new **case-sensitive** `Map` of `Map<String, Map<String, Option<String>>>` type for the struct.
+    ///All values in the Map are stored in `String` type.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -52,15 +164,79 @@ impl Ini {
     ///```
     ///Returns the struct and stores it in the calling variable.
     pub fn new_cs() -> Ini {
+        Ini::new_from_defaults(IniDefault { case_sensitive: true, ..Default::default() })
+    }
+
+    ///Creates a new `Ini` with the given defaults from an existing `IniDefault` object.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///use configparser::ini::IniDefault;
+    ///
+    ///let mut default = IniDefault::default();
+    ///default.comment_symbols = vec![';'];
+    ///default.delimiters = vec!['='];
+    ///let mut config = Ini::new_from_defaults(default.clone());
+    ///// Now, load as usual with new defaults:
+    ///let map = config.load("tests/test.ini").unwrap();
+    ///assert_eq!(config.defaults(), default);
+    ///
+    ///```
+    pub fn new_from_defaults(defaults: IniDefault) -> Ini {
         Ini {
-            map: HashMap::new(),
-            default_section: "default".to_owned(),
-            comment_symbols: vec![';', '#'],
-            delimiters: vec!['=', ':'],
-            case_sensitive: true,
+            map: Map::new(),
+            default_section: defaults.default_section,
+            comment_symbols: defaults.comment_symbols,
+            delimiters: defaults.delimiters,
+            boolean_values: defaults.boolean_values,
+            case_sensitive: defaults.case_sensitive,
+            multiline: defaults.multiline,
         }
     }
 
+    ///Fetches the defaults from the current `Ini` object and stores it as a `IniDefault` struct for usage elsewhere.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///
+    ///let mut config = Ini::new();
+    ///let default = config.defaults();
+    ///```
+    ///Returns an `IniDefault` object. Keep in mind that it will get borrowed since it has non-`Copy` types.
+    pub fn defaults(&self) -> IniDefault {
+        IniDefault {
+            default_section: self.default_section.to_owned(),
+            comment_symbols: self.comment_symbols.to_owned(),
+            delimiters: self.delimiters.to_owned(),
+            boolean_values: self.boolean_values.to_owned(),
+            case_sensitive: self.case_sensitive,
+            multiline: self.multiline,
+        }
+    }
+
+    ///Takes an `IniDefault` object and stores its properties in the calling `Ini` object. This happens in-place and
+    ///does not work retroactively, only future operations are affected.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///use configparser::ini::IniDefault;
+    ///
+    ///let mut config = Ini::new();
+    ///let mut default = IniDefault::default();
+    ///default.case_sensitive = true;
+    ///// This is equivalent to ini_cs() defaults
+    ///config.load_defaults(default.clone());
+    ///assert_eq!(config.defaults(), default);
+    ///```
+    ///Returns nothing.
+    pub fn load_defaults(&mut self, defaults: IniDefault) {
+        self.default_section = defaults.default_section;
+        self.comment_symbols = defaults.comment_symbols;
+        self.delimiters = defaults.delimiters;
+        self.boolean_values = defaults.boolean_values;
+        self.case_sensitive = defaults.case_sensitive;
+    }
+
     ///Sets the default section header to the defined string (the default is `default`).
     ///It must be set before `load()` or `read()` is called in order to take effect.
     ///## Example
@@ -92,7 +268,22 @@ impl Ini {
         self.comment_symbols = symlist.to_vec();
     }
 
-    ///Gets all the sections of the currently-stored `HashMap` in a vector.
+    ///Sets multiline string support.
+    ///It must be set before `load()` or `read()` is called in order to take effect.
+    ///## Example
+    ///```rust
+    ///use configparser::ini::Ini;
+    ///
+    ///let mut config = Ini::new();
+    ///config.set_multiline(true);
+    ///let map = config.load("tests/test.ini").unwrap();
+    ///```
+    ///Returns nothing.
+    pub fn set_multiline(&mut self, multiline: bool) {
+        self.multiline = multiline;
+    }
+
+    ///Gets all the sections of the currently-stored `Map` in a vector.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -107,7 +298,7 @@ impl Ini {
     }
 
     ///Loads a file from a defined path, parses it and puts the hashmap into our struct.
-    ///At one time, it only stores one configuration, so each call to `load()` or `read()` will clear the existing `HashMap`, if present.
+    ///At one time, it only stores one configuration, so each call to `load()` or `read()` will clear the existing `Map`, if present.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -117,33 +308,36 @@ impl Ini {
     /////Then, we can use standard hashmap functions like:
     ///let values = map.get("values").unwrap();
     ///```
-    ///Returns `Ok(map)` with a clone of the stored `HashMap` if no errors are thrown or else `Err(error_string)`.
+    ///Returns `Ok(map)` with a clone of the stored `Map` if no errors are thrown or else `Err(error_string)`.
     ///Use `get_mut_map()` if you want a mutable reference.
-    pub fn load(
+    pub fn load<T: AsRef<Path>>(
         &mut self,
-        path: &str,
-    ) -> Result<HashMap<String, HashMap<String, Option<String>>>, String> {
-        let path = Path::new(path);
-        let display = path.display();
-
-        let mut file = match File::open(&path) {
-            Err(why) => return Err(format!("couldn't open {}: {}", display, why)),
-            Ok(file) => file,
-        };
-
-        let mut s = String::new();
-        self.map = match file.read_to_string(&mut s) {
-            Err(why) => return Err(format!("couldn't read {}: {}", display, why)),
-            Ok(_) => match self.parse(s) {
-                Err(why) => return Err(why),
-                Ok(map) => map,
-            },
+        path: T,
+    ) -> Result<Map<String, Map<String, Option<String>>>, String> {
+        self.map = match self.parse(match fs::read_to_string(&path) {
+            Err(why) => {
+                return Err(format!(
+                    "couldn't read {}: {}",
+                    &path.as_ref().display(),
+                    why
+                ))
+            }
+            Ok(s) => s,
+        }) {
+            Err(why) => {
+                return Err(format!(
+                    "couldn't read {}: {}",
+                    &path.as_ref().display(),
+                    why
+                ))
+            }
+            Ok(map) => map,
         };
         Ok(self.map.clone())
     }
 
     ///Reads an input string, parses it and puts the hashmap into our struct.
-    ///At one time, it only stores one configuration, so each call to `load()` or `read()` will clear the existing `HashMap`, if present.
+    ///At one time, it only stores one configuration, so each call to `load()` or `read()` will clear the existing `Map`, if present.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -158,12 +352,12 @@ impl Ini {
     ///let this_year = map["2000s"]["2020"].clone().unwrap();
     ///assert_eq!(this_year, "bad"); // value accessible!
     ///```
-    ///Returns `Ok(map)` with a clone of the stored `HashMap` if no errors are thrown or else `Err(error_string)`.
+    ///Returns `Ok(map)` with a clone of the stored `Map` if no errors are thrown or else `Err(error_string)`.
     ///Use `get_mut_map()` if you want a mutable reference.
     pub fn read(
         &mut self,
         input: String,
-    ) -> Result<HashMap<String, HashMap<String, Option<String>>>, String> {
+    ) -> Result<Map<String, Map<String, Option<String>>>, String> {
         self.map = match self.parse(input) {
             Err(why) => return Err(why),
             Ok(map) => map,
@@ -186,8 +380,8 @@ impl Ini {
     ///}
     ///```
     ///Returns a `std::io::Result<()>` type dependent on whether the write was successful or not.
-    pub fn write(&self, path: &str) -> std::io::Result<()> {
-        fs::write(path, self.unparse())
+    pub fn write<T: AsRef<Path>>(&self, path: T) -> std::io::Result<()> {
+        fs::write(path.as_ref(), self.unparse())
     }
 
     ///Returns a string with the current configuration formatted with valid ini-syntax. This is always safe since the configuration is validated during
@@ -209,41 +403,59 @@ impl Ini {
 
     ///Private function that converts the currently stored configuration into a valid ini-syntax string.
     fn unparse(&self) -> String {
-        let mut out = String::new();
-        let mut cloned = self.map.clone();
-        if let Some(defaultmap) = cloned.get(&self.default_section) {
-            for (key, val) in defaultmap.iter() {
-                out.push_str(&key);
+        // push key/value pairs in outmap to out string.
+        fn unparse_key_values(
+            out: &mut String,
+            outmap: &Map<String, Option<String>>,
+            multiline: bool,
+        ) {
+            for (key, val) in outmap.iter() {
+                out.push_str(key);
+
                 if let Some(value) = val {
-                    out.push_str("=");
-                    out.push_str(&value);
+                    out.push('=');
+
+                    if multiline {
+                        let mut lines = value.lines();
+
+                        out.push_str(lines.next().unwrap());
+
+                        for line in lines {
+                            out.push_str(LINE_ENDING);
+                            out.push_str("    ");
+                            out.push_str(line);
+                        }
+                    } else {
+                        out.push_str(value);
+                    }
                 }
-                out.push_str("\n");
+
+                out.push_str(LINE_ENDING);
             }
-            cloned.remove(&self.default_section);
         }
-        for (section, secmap) in cloned.iter() {
-            out.push_str(&format!("[{}]", section));
-            out.push_str("\n");
-            for (key, val) in secmap.iter() {
-                out.push_str(&key);
-                if let Some(value) = val {
-                    out.push_str("=");
-                    out.push_str(&value);
-                }
-                out.push_str("\n");
+
+        let mut out = String::new();
+
+        if let Some(defaultmap) = self.map.get(&self.default_section) {
+            unparse_key_values(&mut out, defaultmap, self.multiline);
+        }
+
+        for (section, secmap) in self.map.iter() {
+            if section != &self.default_section {
+                write!(out, "[{}]", section).unwrap();
+                out.push_str(LINE_ENDING);
+                unparse_key_values(&mut out, secmap, self.multiline);
             }
         }
         out
     }
 
-    ///Private function that parses ini-style syntax into a HashMap.
-    fn parse(
-        &self,
-        input: String,
-    ) -> Result<HashMap<String, HashMap<String, Option<String>>>, String> {
-        let mut map: HashMap<String, HashMap<String, Option<String>>> = HashMap::new();
+    ///Private function that parses ini-style syntax into a Map.
+    fn parse(&self, input: String) -> Result<Map<String, Map<String, Option<String>>>, String> {
+        let mut map: Map<String, Map<String, Option<String>>> = Map::new();
         let mut section = self.default_section.clone();
+        let mut current_key: Option<String> = None;
+
         let caser = |val: &str| {
             if self.case_sensitive {
                 val.to_owned()
@@ -251,70 +463,89 @@ impl Ini {
                 val.to_lowercase()
             }
         };
-        for (num, lines) in input.lines().enumerate() {
-            let trimmed = match lines.find(|c: char| self.comment_symbols.contains(&c)) {
-                Some(idx) => lines[..idx].trim(),
-                None => lines.trim(),
+
+        for (num, raw_line) in input.lines().enumerate() {
+            let line = match raw_line.find(|c: char| self.comment_symbols.contains(&c)) {
+                Some(idx) => &raw_line[..idx],
+                None => raw_line,
             };
+
+            let trimmed = line.trim();
+
             if trimmed.is_empty() {
                 continue;
             }
-            match trimmed.find('[') {
-                Some(start) => match trimmed.rfind(']') {
-                    Some(end) => {
-                        section = caser(trimmed[start + 1..end].trim());
-                    }
+
+            match (trimmed.find('['), trimmed.rfind(']')) {
+                (Some(0), Some(end)) => {
+                    section = caser(trimmed[1..end].trim());
+
+                    continue;
+                }
+                (Some(0), None) => {
+                    return Err(format!(
+                        "line {}: Found opening bracket for section name but no closing bracket",
+                        num
+                    ));
+                }
+                _ => {}
+            }
+
+            if line.starts_with(char::is_whitespace) && self.multiline {
+                let key = match current_key.as_ref() {
+                    Some(x) => x,
                     None => {
                         return Err(format!(
-                            "line {}:{}: Found opening bracket for section name but no closing bracket",
-                            num, start
-                        ));
+                            "line {}: Started with indentation but there is no current entry",
+                            num,
+                        ))
                     }
-                },
-                None => match trimmed.find(&self.delimiters[..]) {
-                    Some(delimiter) => match map.get_mut(&section) {
-                        Some(valmap) => {
-                            let key = caser(trimmed[..delimiter].trim());
-                            let value = trimmed[delimiter + 1..].trim().to_owned();
-                            if key.is_empty() {
-                                return Err(format!(
-                                    "line {}:{}: Key cannot be empty",
-                                    num, delimiter
-                                ));
-                            } else {
-                                valmap.insert(key, Some(value));
-                            }
-                        }
-                        None => {
-                            let mut valmap: HashMap<String, Option<String>> = HashMap::new();
-                            let key = caser(trimmed[..delimiter].trim());
-                            let value = trimmed[delimiter + 1..].trim().to_owned();
-                            if key.is_empty() {
-                                return Err(format!(
-                                    "line {}:{}: Key cannot be empty",
-                                    num, delimiter
-                                ));
-                            } else {
-                                valmap.insert(key, Some(value));
-                            }
-                            map.insert(section.clone(), valmap);
-                        }
-                    },
-                    None => match map.get_mut(&section) {
-                        Some(valmap) => {
-                            let key = caser(trimmed);
-                            valmap.insert(key, None);
-                        }
-                        None => {
-                            let mut valmap: HashMap<String, Option<String>> = HashMap::new();
-                            let key = caser(trimmed);
-                            valmap.insert(key, None);
-                            map.insert(section.clone(), valmap);
-                        }
-                    },
-                },
+                };
+
+                let valmap = map.entry(section.clone()).or_insert_with(Map::new);
+
+                let val = valmap
+                    .entry(key.clone())
+                    .or_insert_with(|| Some(String::new()));
+
+                match val {
+                    Some(x) => {
+                        x.push_str(LINE_ENDING);
+                        x.push_str(trimmed);
+                    }
+                    None => {
+                        *val = Some(format!("{}{}", LINE_ENDING, trimmed));
+                    }
+                }
+
+                continue;
+            }
+
+            let valmap = map.entry(section.clone()).or_insert_with(Map::new);
+
+            match trimmed.find(&self.delimiters[..]) {
+                Some(delimiter) => {
+                    let key = caser(trimmed[..delimiter].trim());
+
+                    if key.is_empty() {
+                        return Err(format!("line {}:{}: Key cannot be empty", num, delimiter));
+                    } else {
+                        current_key = Some(key.clone());
+
+                        let value = trimmed[delimiter + 1..].trim().to_owned();
+
+                        valmap.insert(key, Some(value));
+                    }
+                }
+                None => {
+                    let key = caser(trimmed);
+                    current_key = Some(key.clone());
+
+                    valmap.insert(key, None);
+                }
             }
         }
+
         Ok(map)
     }
 
@@ -377,8 +608,8 @@ impl Ini {
     }
 
     ///Parses the stored value from the key stored in the defined section to a `bool`. For ease of use, the function converts the type coerces a match.
-    ///It attempts to case-insenstively find `true`, `yes`, `t`, `y` and `1` to parse it as `True`.
-    ///Similarly it attempts to case-insensitvely find `false`, `no`, `f`, `n` and `0` to parse it as `False`.
+    ///It attempts to case-insenstively find `true`, `yes`, `t`, `y`, `1` and `on` to parse it as `True`.
+    ///Similarly it attempts to case-insensitvely find `false`, `no`, `f`, `n`, `0` and `off` to parse it as `False`.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -397,9 +628,21 @@ impl Ini {
                 Some(val) => match val {
                     Some(inner) => {
                         let boolval = &inner.to_lowercase()[..];
-                        if ["true", "yes", "t", "y", "1"].contains(&boolval) {
+                        if self
+                            .boolean_values
+                            .get(&true)
+                            .unwrap()
+                            .iter()
+                            .any(|elem| elem == boolval)
+                        {
                             Ok(Some(true))
-                        } else if ["false", "no", "f", "n", "0"].contains(&boolval) {
+                        } else if self
+                            .boolean_values
+                            .get(&false)
+                            .unwrap()
+                            .iter()
+                            .any(|elem| elem == boolval)
+                        {
                             Ok(Some(false))
                         } else {
                             Err(format!(
@@ -503,7 +746,7 @@ impl Ini {
         }
     }
 
-    ///Returns a clone of the `HashMap` stored in our struct.
+    ///Returns a clone of the `Map` stored in our struct.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -516,8 +759,8 @@ impl Ini {
     ///assert_eq!(map, *config.get_map_ref());  // the cloned map is basically a snapshot that you own
     ///```
     ///Returns `Some(map)` if map is non-empty or else returns `None`.
-    ///Similar to `load()` but returns an `Option` type with the currently stored `HashMap`.
-    pub fn get_map(&self) -> Option<HashMap<String, HashMap<String, Option<String>>>> {
+    ///Similar to `load()` but returns an `Option` type with the currently stored `Map`.
+    pub fn get_map(&self) -> Option<Map<String, Map<String, Option<String>>>> {
         if self.map.is_empty() {
             None
         } else {
@@ -525,7 +768,7 @@ impl Ini {
         }
     }
 
-    ///Returns an immutable reference to the `HashMap` stored in our struct.
+    ///Returns an immutable reference to the `Map` stored in our struct.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -539,11 +782,11 @@ impl Ini {
     ///```
     ///If you just need to definitely mutate the map, use `get_mut_map()` instead. Alternatively, you can generate a snapshot by getting a clone
     ///with `get_map()` and work with that.
-    pub fn get_map_ref(&self) -> &HashMap<String, HashMap<String, Option<String>>> {
+    pub fn get_map_ref(&self) -> &Map<String, Map<String, Option<String>>> {
         &self.map
     }
 
-    ///Returns a mutable reference to the `HashMap` stored in our struct.
+    ///Returns a mutable reference to the `Map` stored in our struct.
     ///## Example
     ///```rust
     ///use configparser::ini::Ini;
@@ -557,11 +800,11 @@ impl Ini {
     ///assert_eq!(config.get("topsecrets", "nuclear launch codes"), None);  // inserted successfully!
     ///```
     ///If you just need to access the map without mutating, use `get_map_ref()` or make a clone with `get_map()` instead.
-    pub fn get_mut_map(&mut self) -> &mut HashMap<String, HashMap<String, Option<String>>> {
+    pub fn get_mut_map(&mut self) -> &mut Map<String, Map<String, Option<String>>> {
         &mut self.map
     }
 
-    ///Sets an `Option<String>` in the `HashMap` stored in our struct. If a particular section or key does not exist, it will be automatically created.
+    ///Sets an `Option<String>` in the `Map` stored in our struct. If a particular section or key does not exist, it will be automatically created.
     ///An existing value in the map  will be overwritten. You can also set `None` safely.
     ///## Example
     ///```rust
@@ -588,7 +831,7 @@ impl Ini {
         match self.map.get_mut(&section) {
             Some(secmap) => secmap.insert(key, value),
             None => {
-                let mut valmap: HashMap<String, Option<String>> = HashMap::new();
+                let mut valmap: Map<String, Option<String>> = Map::new();
                 valmap.insert(key, value);
                 self.map.insert(section, valmap);
                 None
@@ -596,7 +839,7 @@ impl Ini {
         }
     }
 
-    ///Sets an `Option<&str>` in the `HashMap` stored in our struct. If a particular section or key does not exist, it will be automatically created.
+    ///Sets an `Option<&str>` in the `Map` stored in our struct. If a particular section or key does not exist, it will be automatically created.
     ///An existing value in the map  will be overwritten. You can also set `None` safely.
     ///## Example
     ///```rust
@@ -640,6 +883,7 @@ impl Ini {
     }
 
     ///Removes a section from the hashmap, returning the properties stored in the section if the section was previously in the map.
+    ///## Example
     ///```rust
     ///use configparser::ini::Ini;
     ///
@@ -651,7 +895,7 @@ impl Ini {
     ///assert!(config.get_map_ref().is_empty());  // with the last section removed, our map is now empty!
     ///```
     ///Returns `Some(section_map)` if the section exists or else, `None`.
-    pub fn remove_section(&mut self, section: &str) -> Option<HashMap<String, Option<String>>> {
+    pub fn remove_section(&mut self, section: &str) -> Option<Map<String, Option<String>>> {
         let section = if self.case_sensitive {
             section.to_owned()
         } else {
@@ -661,6 +905,7 @@ impl Ini {
     }
 
     ///Removes a key from a section in the hashmap, returning the value attached to the key if it was previously in the map.
+    ///## Example
     ///```rust
     ///use configparser::ini::Ini;
     ///
@@ -679,3 +924,49 @@ impl Ini {
         self.map.get_mut(&section)?.remove(&key)
     }
 }
+
+#[cfg(feature = "async-std")]
+impl Ini {
+    ///Loads a file asynchronously from a defined path, parses it and puts the hashmap into our struct.
+    ///At one time, it only stores one configuration, so each call to `load()` or `read()` will clear the existing `Map`, if present.
+    ///
+    ///Usage is similar to `load`, but `.await` must be called after along with the usual async rules.
+    ///
+    ///Returns `Ok(map)` with a clone of the stored `Map` if no errors are thrown or else `Err(error_string)`.
+    ///Use `get_mut_map()` if you want a mutable reference.
+    pub async fn load_async<T: AsRef<AsyncPath>>(
+        &mut self,
+        path: T,
+    ) -> Result<Map<String, Map<String, Option<String>>>, String> {
+        self.map = match self.parse(match async_fs::read_to_string(&path).await {
+            Err(why) => {
+                return Err(format!(
+                    "couldn't read {}: {}",
+                    &path.as_ref().display(),
+                    why
+                ))
+            }
+            Ok(s) => s,
+        }) {
+            Err(why) => {
+                return Err(format!(
+                    "couldn't read {}: {}",
+                    &path.as_ref().display(),
+                    why
+                ))
+            }
+            Ok(map) => map,
+        };
+        Ok(self.map.clone())
+    }
+
+    ///Writes the current configuation to the specified path asynchronously. If a file is not present, it is automatically created for you, if a file already
+    ///exists, it is truncated and the configuration is written to it.
+    ///
+    ///Usage is the same as `write`, but `.await` must be called after along with the usual async rules.
+    ///
+    ///Returns a `std::io::Result<()>` type dependent on whether the write was successful or not.
+    pub async fn write_async<T: AsRef<Path>>(&self, path: T) -> std::io::Result<()> {
+        async_fs::write(path.as_ref(), self.unparse()).await
+    }
+}
